
/*
 * This file is part of gps-data-logger-widget
 *
 * Copyright (C) 2009-2011 Marcell Lengyel <marcell@maemo.org>
 *
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>	/*  getenv */
#include <time.h>
#include <locale.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glib/gprintf.h>

#include <glib/gi18n-lib.h> /* Declares _(). */

#include <hildon/hildon.h>
#include <libhildondesktop/libhildondesktop.h>
#include <hildon/hildon-banner.h>

/* screen on/off */
#include <dbus/dbus.h>
#include <dbus/dbus-glib-lowlevel.h> /* dbus_connection_setup_with_g_main */
#include <mce/dbus-names.h>
#include <mce/mode-names.h>

#include <location/location-distance-utils.h> /* location_distance_between */
#include "gps-data-logger-widget.h"
#include "gps-data-logger-gfx.h"
#include "settings-dialog.h"

#define OSSO_GDL_NAME    "gps_data_logger"
#define OSSO_GDL_SERVICE "org.maemo."OSSO_GDL_NAME
#define OSSO_GDL_OBJECT  "/org/maemo/"OSSO_GDL_NAME
#define OSSO_GDL_IFACE   "org.maemo."OSSO_GDL_NAME

#define MCE_MATCH_RULE "type='signal',interface='" MCE_SIGNAL_IF \
	                       "',member='" MCE_DISPLAY_SIG "'"

HD_DEFINE_PLUGIN_MODULE (GpsDataLoggerWidget, gps_data_logger_widget, HD_TYPE_HOME_PLUGIN_ITEM)

static void update_status_and_buttons (GpsDataLoggerWidget * self);
static gboolean manual_on_stop (gpointer data);
static void on_stop(LocationGPSDControl *control, gpointer data);
void gps_data_logger_update_distance(gpointer data);
	
/* -------------------------------------------------------------------- */

static gboolean
on_style_set(GtkWidget *widget, GtkStyle *previous_style, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);

    /* if pixmaps already loaded, free them */
    if (theme_pixbufs_loaded(self))
            free_theme_pixbufs(self);

    load_theme_pixbufs(self);

    if (!theme_pixbufs_loaded(self))
            free_theme_pixbufs(self);

    return FALSE;
}

static void
free_icons(GpsDataLoggerWidget * self)
{
        free_pixbuf(&self->rec_icon);
        free_pixbuf(&self->stop_icon);
        free_pixbuf(&self->pause_icon);
	free_pixbuf(&self->gps_refresh_icon);
	free_pixbuf(&self->gps_on_icon);
	free_pixbuf(&self->gps_off_icon);
}

static void
load_icons(GpsDataLoggerWidget * self)
{
        GtkIconTheme *theme;

        theme = gtk_icon_theme_get_default();

        /* Load icons */
        self->rec_icon = gtk_icon_theme_load_icon(theme,
                        REC_ICON, 48, 0, NULL);
        self->stop_icon = gtk_icon_theme_load_icon(theme,
                        STOP_ICON, 48, 0, NULL);
        self->pause_icon = gtk_icon_theme_load_icon(theme,
                        PAUSE_ICON, 48, 0, NULL);
        self->gps_refresh_icon = gtk_icon_theme_load_icon(theme,
                        GPS_REFRESH_ICON, 48, 0, NULL);
        self->gps_on_icon = gtk_icon_theme_load_icon(theme,
                        GPS_ON_ICON, 48, 0, NULL);
        self->gps_off_icon = gtk_icon_theme_load_icon(theme,
                        GPS_OFF_ICON, 48, 0, NULL);
}

/* -------------------------------------------------------------------- */

/* DEBUG log */

#ifdef DEBUG
static void
debuglog_close(GpsDataLoggerWidget * self)
{
    if (self->logfile) {
	fclose(self->logfile);
	self->logfile = NULL;
    }
}
#endif

#ifdef DEBUG
static void
debuglog_start(GpsDataLoggerWidget * self)
{
    gchar *path;
    time_t current;
    struct tm *tp;


    /* If the logfile is already open (should not) then close it */
    if (self->logfile)
        debuglog_close(self);
    /* Name the log file using the current time */
    current = time(NULL);
    tp = localtime(&current);
    path = g_strdup_printf("%s/%d%02d%02d_%02d%02d%02d.log", 
		           self->settings->log_folder,
		    	   tp->tm_year + 1900,
		   	   tp->tm_mon + 1,
		   	   tp->tm_mday,
		   	   tp->tm_hour,
		   	   tp->tm_min,
		   	   tp->tm_sec );
    self->logfile = fopen(path, "w");
    g_free(path);
    
}
#endif

/* -------------------------------------------------------------------- */

/* GPX file */

static void
logfile_close(GpsDataLoggerWidget * self)
{
    if (self->filtered_file) {
        g_fprintf(self->filtered_file, "</trkseg>\n</trk>\n</gpx>\n");
        fclose(self->filtered_file);
        self->filtered_file = NULL;
    }
    if (self->file) {
        g_fprintf(self->file, "</trkseg>\n</trk>\n</gpx>\n");
        fclose(self->file);
        self->file = NULL;
    }
    if (self->path) {
        g_free(self->path);
	self->path = NULL;
    }
}

static void
logfile_start(GpsDataLoggerWidget * self)
{
    /* gchar *path; */
    time_t current;
    struct tm *tp;
    char string_buf[5];
    char *filtered_path;

    /* logfile_start can only be called in APP_STATE_STARTING state */
    if (self->app_state != APP_STATE_STARTING)
	return;
    /* Test locale if it has a comma for the decimal point */
    g_snprintf(string_buf, 5, "%f", 3.14);
    if (g_strrstr(string_buf,",") != NULL) {
        self->has_comma_as_decimal_point = TRUE;
    } else {
        self->has_comma_as_decimal_point = FALSE;
    }
    /* If the logfile is already open (should not happen) then close it */
    if ((self->file) || (self->filtered_file))
        logfile_close(self);
    /* Name the log file using the current time */
    current = time(NULL);
    tp = localtime(&current);

    if (self->settings->save_filtered_log) {
	    filtered_path = g_strdup_printf("%s/%d%02d%02d_%02d%02d%02d_filtered.gpx",
			           self->settings->log_folder,
			    	   tp->tm_year + 1900,
		   		   tp->tm_mon + 1,
			   	   tp->tm_mday,
			   	   tp->tm_hour,
			   	   tp->tm_min,
			   	   tp->tm_sec );
	    if (! filtered_path) {
        	/* out of memory?? */
	        hildon_banner_show_informationf(NULL,
        	                NULL,
                	        _("Could not open the filtered GPX file for writing!"));
		self->filtered_file = NULL;
	    } else {
    		self->filtered_file = fopen(filtered_path, "w");
	    	if (! (self->filtered_file)) {
        		hildon_banner_show_informationf(NULL,
                	        NULL,
                        	_("Could not open \"%s\" for writing!"),
				filtered_path);
#ifndef __arm__
			g_print(_("Could not open the filtered file for writing\n"));
#endif
		} else {
        		g_fprintf(self->filtered_file, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	        	g_fprintf(self->filtered_file, "<gpx version=\"1.0\"\n");
		        g_fprintf(self->filtered_file, "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
		        g_fprintf(self->filtered_file, "creator=\"gps-data-logger on Maemo\"\n");
		        g_fprintf(self->filtered_file, "xmlns=\"http://www.topografix.com/GPX/1/0\"\n");
		        g_fprintf(self->filtered_file, "xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd\">\n");
		        g_fprintf(self->filtered_file, "<trk>\n");
		        g_fprintf(self->filtered_file, "<trkseg>\n");
		}
	    }
    } else {
	self->filtered_file = NULL;
    }
	    
    if (self->path) {
	g_free(self->path);
    }
    self->path = g_strdup_printf("%s/%d%02d%02d_%02d%02d%02d.gpx", 
		           self->settings->log_folder,
		    	   tp->tm_year + 1900,
		   	   tp->tm_mon + 1,
		   	   tp->tm_mday,
		   	   tp->tm_hour,
		   	   tp->tm_min,
		   	   tp->tm_sec );
    if (! self->path) {
        /* out of memory?? */
        self->app_state = APP_STATE_IDLE;
        location_gpsd_control_stop(self->control);
        update_status_and_buttons(self);
        hildon_banner_show_informationf(NULL,
                        NULL,
                        _("Could not open the GPX file for writing!"));
	return;
    }
    self->file = fopen(self->path, "w");
    if (! (self->file)) {
        hildon_banner_show_informationf(NULL,
                        NULL,
                        _("Could not open \"%s\" for writing!"),
			self->path);
#ifndef __arm__
	g_print(_("Could not open logfile for writing, going to IDLE\n"));
#endif
        self->app_state = APP_STATE_IDLE;
        location_gpsd_control_stop(self->control);
        update_status_and_buttons(self);
    } else {
        g_fprintf(self->file, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        g_fprintf(self->file, "<gpx version=\"1.0\"\n");
        g_fprintf(self->file, "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n");
        g_fprintf(self->file, "creator=\"gps-data-logger on Maemo\"\n");
        g_fprintf(self->file, "xmlns=\"http://www.topografix.com/GPX/1/0\"\n");
        g_fprintf(self->file, "xsi:schemaLocation=\"http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd\">\n");
        g_fprintf(self->file, "<trk>\n");
        g_fprintf(self->file, "<trkseg>\n");
    }
    /* g_free(path); */
}

/* -------------------------------------------------------------------- */

static gboolean
on_widget_leave(GtkWidget* widget, GdkEventCrossing *event, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);

    self->rec_button_down = FALSE;
    self->stop_button_down = FALSE;
    gtk_widget_queue_draw(widget);

    return FALSE;
}

static void
update_status_and_buttons (GpsDataLoggerWidget * self)
{
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- enter (state: %d)\n", __func__, self->app_state);
	fflush(self->logfile);
    }
#endif

    switch(self->app_state) {
	case APP_STATE_STARTING:
		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
				           self->gps_refresh_icon);
		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
		self->rec_button_enabled = FALSE;
		self->stop_button_enabled = TRUE;
		self->distance = 0.0;
#ifndef __arm__
		g_print(" starting: loading new icon, disabling rec\n");
#endif
	        /* if the screen is on, update the distance counter */
	        if (self->display_status)
	            gps_data_logger_update_distance(self);	

		break;
	case APP_STATE_RECORDING:
		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
				           self->has_fix ? self->gps_on_icon : self->gps_refresh_icon);
		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
		self->pause_button_visible = TRUE;
		self->rec_button_enabled = TRUE;
		self->stop_button_enabled = TRUE;
		break;
	case APP_STATE_PAUSED:
		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
				           self->has_fix ? self->gps_on_icon : self->gps_refresh_icon);
		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
		self->pause_button_visible = FALSE;
		break;
	case APP_STATE_AUTO_PAUSED:
		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
				           self->has_fix ? self->gps_on_icon : self->gps_refresh_icon);
		gtk_image_set_from_pixbuf (GTK_IMAGE(self->auto_pause_status),
				           self->pause_icon);
		break;
	case APP_STATE_STOPPING:
		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
				           self->gps_off_icon);
		self->pause_button_visible = FALSE;
		self->rec_button_enabled = FALSE;
		self->stop_button_enabled = FALSE;
		break;
	case APP_STATE_IDLE:
	default:
		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
				           self->gps_off_icon);
		gtk_image_clear(GTK_IMAGE(self->auto_pause_status));
		self->pause_button_visible = FALSE;
		self->rec_button_enabled = TRUE;
		self->stop_button_enabled = FALSE;
#ifndef __arm__
		g_print(" idle or default\n");
#endif
		break;
    }
    gtk_widget_queue_draw(GTK_WIDGET(self));
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- leave\n", __func__);
	fflush(self->logfile);
    }
#endif
}

static void
handle_button_press (GtkWidget *button,
		     gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- enter (state: %d)\n", __func__, self->app_state);
	fflush(self->logfile);
    }
#endif

    if (button == self->button_rec) {
        if (self->app_state == APP_STATE_IDLE) {
            self->app_state = APP_STATE_STARTING;
#ifndef __arm__
	    g_print(" new state: starting\n");
            g_print("handle_button_press : calling update_status_and_buttons\n");
#endif
            update_status_and_buttons(self);
	    /* open a new log file */
	    logfile_start(self);
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- calling location_gpsd_control_start\n", __func__);
	fflush(self->logfile);
    }
#endif
            location_gpsd_control_start(self->control);
	    g_usleep(500);
	    location_gps_device_reset_last_known(self->device);
	}
	else if ((self->app_state == APP_STATE_RECORDING) ||
		 (self->app_state == APP_STATE_AUTO_PAUSED)) {
            self->app_state = APP_STATE_PAUSED;
#ifndef __arm__
	    g_print(" new state: paused\n");
            g_print("handle_button_press : calling update_status_and_buttons\n");
#endif
            update_status_and_buttons(self);
            location_gpsd_control_stop(self->control);
	}
	else if (self->app_state == APP_STATE_PAUSED) {
            self->app_state = APP_STATE_STARTING;
#ifndef __arm__
	    g_print(" new state: starting, after pause\n");
            g_print("handle_button_press : calling update_status_and_buttons\n");
#endif
            update_status_and_buttons(self);
	    /* close tarck segment and start a new one,
	     * if there were points recorded in the previous
	     * segment */
	    if (self->points_recorded_in_current_segment)
	        g_fprintf(self->file, "</trkseg>\n<trkseg>\n");
	    if (self->filtered_points_recorded_in_current_segment)
	        g_fprintf(self->filtered_file, "</trkseg>\n<trkseg>\n");
	    location_gps_device_reset_last_known(self->device);
            location_gpsd_control_start(self->control);
	}
    } else if (button == self->button_stop) {
	if (self->app_state == APP_STATE_PAUSED) {
	    /* If the state is changing from paused->stopped
	     * then the gps is already stopped, so no on_stop
	     * signal will come. Set a timer and call it. */
	    g_timeout_add_seconds(2,
			          manual_on_stop,
				  data);

	}
	self->app_state = APP_STATE_STOPPING;
#ifndef __arm__
	g_print(" new state: stoping\n");
        g_print("handle_button_press : calling update_status_and_buttons\n");
#endif
        update_status_and_buttons(self);
        location_gpsd_control_stop(self->control);
	/* close log file */
	logfile_close(self);
    }
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- leave (new state: %d)\n", __func__, self->app_state);
	fflush(self->logfile);
    }
#endif
}

static gboolean
on_button_release(GtkWidget* widget, GdkEventButton *button_event, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
    gboolean retval = FALSE;

#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- enter ", __func__);
        if (widget == GTK_WIDGET(self->background)) {
	    g_fprintf(self->logfile, " widget: background\n");
	} else if (widget == GTK_WIDGET(self->button_rec)) {
	    g_fprintf(self->logfile, " widget: button_rec\n");
	} else if (widget == GTK_WIDGET(self->button_stop)) {
	    g_fprintf(self->logfile, " widget: button_stop\n");
	}
	fflush(self->logfile);
    }
#endif
    self->rec_button_down = FALSE;
    self->stop_button_down = FALSE;
    gtk_widget_queue_draw(GTK_WIDGET(self));
    g_usleep(500);

    if (widget == GTK_WIDGET(self->background)) {
        if (button_event->x >= self->button_rec->allocation.x &&
            button_event->y >= self->button_rec->allocation.y &&
            button_event->x <= self->button_rec->allocation.x +
                           self->button_rec->allocation.width &&
            button_event->y <= self->button_rec->allocation.y +
                           self->button_rec->allocation.height &&
            self->rec_button_enabled) {
#ifdef DEBUG
                if (self->logfile) {
                    g_fprintf(self->logfile, "  calling handle_button_press on background for button_rec\n");
	            fflush(self->logfile);
		}
#endif
                handle_button_press(self->button_rec, self);
        } else if (button_event->x >= self->button_stop->allocation.x &&
            button_event->y >= self->button_stop->allocation.y &&
            button_event->x <= self->button_stop->allocation.x +
                           self->button_stop->allocation.width &&
            button_event->y <= self->button_stop->allocation.y +
                           self->button_stop->allocation.height &&
            self->stop_button_enabled) {
#ifdef DEBUG
                if (self->logfile) {
                    g_fprintf(self->logfile, "  calling handle_button_press on background for button_stop\n");
	            fflush(self->logfile);
		}
#endif
                handle_button_press(self->button_stop, self);
        }	
    } else if ((widget == GTK_WIDGET(self->button_rec)) && 
		    self->rec_button_enabled) {
#ifdef DEBUG
        if (self->logfile) {
            g_fprintf(self->logfile, "  calling handle_button_press on button_rec for button_rec\n");
	    fflush(self->logfile);
	}
#endif
        handle_button_press(self->button_rec, self);
    } else if ((widget == GTK_WIDGET(self->button_stop)) && 
		    self->stop_button_enabled) {
#ifdef DEBUG
        if (self->logfile) {
             g_fprintf(self->logfile, "  calling handle_button_press on button_stop for button_stop\n");
	     fflush(self->logfile);
	}
#endif
        handle_button_press(self->button_stop, self);
    }
    
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- leave\n", __func__);
	fflush(self->logfile);
    }
#endif
    return retval;
}

static gboolean
on_button_press(GtkWidget* widget, GdkEventButton *button_event, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
    gboolean retval = FALSE;

#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- enter ", __func__);
        if (widget == GTK_WIDGET(self->background)) {
	    g_fprintf(self->logfile, " widget: background\n");
	} else if (widget == GTK_WIDGET(self->button_rec)) {
	    g_fprintf(self->logfile, " widget: button_rec\n");
	} else if (widget == GTK_WIDGET(self->button_stop)) {
	    g_fprintf(self->logfile, " widget: button_stop\n");
	}
	fflush(self->logfile);
    }
#endif
    if (widget == GTK_WIDGET(self->background)) {
        if (button_event->x >= self->button_rec->allocation.x &&
           button_event->y >= self->button_rec->allocation.y &&
            button_event->x <= self->button_rec->allocation.x +
                           self->button_rec->allocation.width &&
            button_event->y <= self->button_rec->allocation.y +
                           self->button_rec->allocation.height &&
            self->rec_button_enabled)
        {
            self->rec_button_down = TRUE;
            gtk_widget_queue_draw(widget);
        } else if (button_event->x >= self->button_stop->allocation.x &&
            button_event->y >= self->button_stop->allocation.y &&
            button_event->x <= self->button_stop->allocation.x +
                           self->button_stop->allocation.width &&
            button_event->y <= self->button_stop->allocation.y +
                           self->button_stop->allocation.height &&
            self->stop_button_enabled)
        {
            self->stop_button_down = TRUE;
            gtk_widget_queue_draw(widget);
        }
    } else if ((widget == GTK_WIDGET(self->button_rec)) &&
		    self->rec_button_enabled) {
        self->rec_button_down = TRUE;
        gtk_widget_queue_draw(widget);
    } else if ((widget == GTK_WIDGET(self->button_stop)) &&
		    self->stop_button_enabled) {
        self->stop_button_down = TRUE;
        gtk_widget_queue_draw(widget);
    }

#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- leave\n", __func__);
	fflush(self->logfile);
    }
#endif
    return retval;
}

void gps_data_logger_update_distance(gpointer data)
{
	GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
	gchar *dist = NULL;

#ifdef DEBUG
        if (self->logfile) {
            g_fprintf(self->logfile, "%s -- enter, distance: %.2f km\n", __func__, self->distance);
	    fflush(self->logfile);
        }
#endif
	/* do this only if the widget is visible! */
	dist = g_malloc(25);
	if (self->distance < 1) {
	    g_snprintf(dist, 25, "%.0f m", self->distance * 1000);
	} else if (self->distance < 10) {
	    g_snprintf(dist, 25, "%.2f km", self->distance);
	} else {
	    g_snprintf(dist, 25, "%.1f km", self->distance);
	}
	gtk_label_set_text(GTK_LABEL(self->distance_value_string), dist);
}

/* -------------------------------------------------------------------- */
/* liblocation */

static void
on_error(LocationGPSDControl *control, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
#ifndef __arm__
    g_printf("%s\n", __func__);
#endif
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- enter\n", __func__);
	fflush(self->logfile);
    }
#endif
    if (self->app_state != APP_STATE_IDLE) {
	hildon_banner_show_information(NULL,
			NULL,
			_("on_error, going to IDLE state"));
	self->app_state = APP_STATE_IDLE;
	location_gpsd_control_stop(self->control);
#ifndef __arm__
        g_print("on_error : calling update_status_and_buttons\n");
#endif
	update_status_and_buttons(self);
	logfile_close(self);
    }
}

static void
on_start(LocationGPSDControl *control, gpointer data)
{
#ifndef __arm__
        g_printf("%s\n", __func__);
#endif
#ifdef DEBUG
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- was here\n", __func__);
	fflush(self->logfile);
    }
#endif
}

static gboolean
manual_on_stop(gpointer data)
{
    /* call on_stop manually */
    on_stop(NULL, data);
    return FALSE;
}

static void
on_stop(LocationGPSDControl *control, gpointer data)
{
	GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
#ifndef __arm__
        g_printf("%s\n", __func__);
#endif
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- enter\n", __func__);
	fflush(self->logfile);
    }
#endif
	if (self->app_state == APP_STATE_STOPPING) {
	    if (self->path) {
        	hildon_banner_show_informationf(NULL,
			                       NULL,
		        	               _("on_stop, going to IDLE state\nLog saved to \"%s\""),
					       self->path);
	    } else {
        	hildon_banner_show_information(NULL,
			                       NULL,
		        	               _("on_stop, going to IDLE state"));
	    }
	    self->app_state = APP_STATE_IDLE;
#ifndef __arm__
            g_print("on_stop : calling update_status_and_buttons\n");
#endif
#ifdef DEBUG
            if (self->logfile) {
                g_fprintf(self->logfile, "%s -- going to IDLE, calling update_status_and_buttons\n", __func__);
	        fflush(self->logfile);
            }
#endif
		update_status_and_buttons(self);
	}
}


static void
on_changed(LocationGPSDevice *device, gpointer data)
{
	GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
	time_t tme;
	struct tm *tp;
	char string_buf[256];
	int i;
	
#ifdef DEBUG
        if (self->logfile) {
            g_fprintf(self->logfile, "%s -- enter, state: %d\n", __func__, self->app_state);
	    fflush(self->logfile);
        }
#endif
        if (!device)
                return;
	/* if there is no fix, or we are recording, but the fix is so
	 * bad that it is a 2D fix only, with no time set then show
	 * the refresh icon and autopause if that is allowed. */
	if ((device->status == LOCATION_GPS_DEVICE_STATUS_NO_FIX) ||
	    ((device->fix) && (self->app_state == APP_STATE_RECORDING) &&
	     (! device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET) &&
	     (device->fix->mode == LOCATION_GPS_DEVICE_MODE_2D))) {
		/* fix is lost */
		self->has_fix = FALSE;
		/* if auto_pause is enabled, let's engage that too */
		if ((self->app_state == APP_STATE_RECORDING) &&
		    (self->settings->auto_pause))
			self->app_state = APP_STATE_AUTO_PAUSED;
		update_status_and_buttons(self);
	} else if (device->fix) {
#ifdef DEBUG
                if (self->logfile) {
                    g_fprintf(self->logfile, "  fix fiels: %d, fix mode: %d, accuracy (m): %.0f\n", device->fix->fields, device->fix->mode, device->fix->eph / 100);
	            fflush(self->logfile);
                }
#endif
		if ((! self->has_fix) && 
		    (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET)) {
		    /* fix found again */
		    self->has_fix = TRUE;
		    update_status_and_buttons(self);
		}
		if (((self->app_state == APP_STATE_STARTING) ||
		     (self->app_state == APP_STATE_RECORDING) ||
		     (self->app_state == APP_STATE_AUTO_PAUSED)) &&
		    (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET) &&
		    (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET)) {
			    /* Shall we stay in auto_paused state ? */
			if (((self->app_state == APP_STATE_AUTO_PAUSED) &&
			     (device->fix->fields & LOCATION_GPS_DEVICE_SPEED_SET) &&
			     (device->fix->speed >= self->settings->pause_speed_threshold) ) ||
			   /* or just started */
			   (self->app_state == APP_STATE_STARTING)) {
			    hildon_banner_show_information(NULL,
                                NULL,
                                _("on_changed, going to RECORDING state"));
                  
                            self->app_state = APP_STATE_RECORDING;
			    self->prev_lat = 0.0;
			    self->prev_lon = 0.0;
#ifndef __arm__
                            g_print("on_change : calling update_status_and_buttons\n");
#endif
                            update_status_and_buttons(self);
	                    self->points_recorded_in_current_segment = 0;
	                    self->filtered_points_recorded_in_current_segment = 0;
			    self->auto_pause_time_mark = 0;
			} else if ((self->app_state == APP_STATE_RECORDING) &&  /* self->app_state == APP_STATE_STARTING  or keep auto_pause? */
				   (self->settings->auto_pause) &&
				   (device->fix->fields & LOCATION_GPS_DEVICE_SPEED_SET) &&
				   (device->fix->speed < self->settings->pause_speed_threshold)) {
			    /* RECORDING, shall we auto_pause? */
				if ((self->auto_pause_time_mark == 0) &&
				    (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET)) {
					self->auto_pause_time_mark = device->fix->time;
				} else if (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET) {
					if ((device->fix->time - self->auto_pause_time_mark) >=
					    (self->settings->pause_time_threshold * AUTO_PAUSE_TIME_MULTIPLIER)) {
					    /* auto_pause speed & time thresholds reached, do auto_pause */
					    self->app_state = APP_STATE_AUTO_PAUSED;
					    update_status_and_buttons(self);
					    return;
					}
				}
					
			    
			}
			
			tme = device->fix->time;
			tp = gmtime(&tme);
			g_snprintf(string_buf, 256, "<trkpt lat=\"%f\" lon=\"%f\">\n",
					device->fix->latitude,
					device->fix->longitude);
			if (self->has_comma_as_decimal_point) {
			    for (i = 0; i < sizeof(string_buf); i++) {
				if (string_buf[i] == ',')
					string_buf[i] = '.';
			    }
			}
			g_fprintf(self->file, string_buf);
			/* calculate distance */
			if (self->prev_lat != 0.0 && self->prev_lon != 0.0
					&& self->app_state == APP_STATE_RECORDING) {
			    /*
			    self->distance += location_distance_between(self->prev_lat,
									self->prev_lon,
									device->fix->latitude,
									device->fix->longitude);
									*/
			    double delta;
			    delta = location_distance_between(self->prev_lat,
									self->prev_lon,
									device->fix->latitude,
									device->fix->longitude);
			    /* if the delta is smaller then the possible GPS error, drop
			     * the data, and use the previous prev_lat and prev_lon */
			    if ((delta * 1000) > (device->fix->eph / 100.0)) {
			        self->distance += delta;
				self->prev_lat = device->fix->latitude;
				self->prev_lon = device->fix->longitude;

			        /* if the screen is on, update the distance counter */
			        if (self->display_status)
			            gps_data_logger_update_distance(self);
			        /* save the filtered log if needed */
			        if (self->settings->save_filtered_log) {
			            self->filtered_points_recorded_in_current_segment += 1;
			            g_fprintf(self->filtered_file, string_buf);
				    g_fprintf(self->filtered_file, 
					      "  <time>%d-%02d-%02dT%02d:%02d:%02dZ</time>\n",
					      tp->tm_year + 1900,
					      tp->tm_mon + 1,
					      tp->tm_mday,
					      tp->tm_hour,
					      tp->tm_min,
					      tp->tm_sec);
				    g_fprintf(self->filtered_file, "</trkpt>\n");
				}	
			    }
#ifdef DEBUG
                            if (self->logfile) {
                                g_fprintf(self->logfile, "  distance: %f, delta: %f\n", 
						self->distance, delta);
	                        fflush(self->logfile);
                            }
#endif

			} else {
			    self->prev_lat = device->fix->latitude;
			    self->prev_lon = device->fix->longitude;
			}
			if (device->fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET) {
			    g_snprintf(string_buf, 256, "  <ele>%.1f</ele>\n",
						device->fix->altitude);
			    if (self->has_comma_as_decimal_point) {
				for (i=0; i<sizeof(string_buf); i++) {
					if (string_buf[i] == ',')
						string_buf[i] = '.';
				}
			    }
			    g_fprintf(self->file, string_buf);
			}
			g_fprintf(self->file, 
				  "  <time>%d-%02d-%02dT%02d:%02d:%02dZ</time>\n",
				  tp->tm_year + 1900,
				  tp->tm_mon + 1,
				  tp->tm_mday,
				  tp->tm_hour,
				  tp->tm_min,
				  tp->tm_sec);
			if (device->fix->mode == LOCATION_GPS_DEVICE_MODE_2D)
				g_fprintf(self->file, "  <fix>2d</fix>\n");
			else if (device->fix->mode == LOCATION_GPS_DEVICE_MODE_3D)
				g_fprintf(self->file, "  <fix>3d</fix>\n");
			g_fprintf(self->file, "  <sat>%d</sat>\n",
					device->satellites_in_use);
			/* eph is in cm */
			g_snprintf(string_buf, 256, "  <hdop>%.2f</hdop>\n",
						device->fix->eph / 100.0);
			if (self->has_comma_as_decimal_point) {
			    for (i=0; i<sizeof(string_buf); i++) {
			        if (string_buf[i] == ',')
			            string_buf[i] = '.';
			    }
			}
			g_fprintf(self->file, string_buf);
			g_snprintf(string_buf, 256, "  <vdop>%.2f</vdop>\n",
						device->fix->epv);
			if (self->has_comma_as_decimal_point) {
			    for (i=0; i<sizeof(string_buf); i++) {
			        if (string_buf[i] == ',')
			            string_buf[i] = '.';
			    }
			}
			g_fprintf(self->file, string_buf);
			g_fprintf(self->file, "</trkpt>\n");
			self->points_recorded_in_current_segment += 1;
		} /* STARING & RECORDING & has coordinates */
#ifndef __arm__
                if (device->fix->fields & LOCATION_GPS_DEVICE_TIME_SET)
                        g_printf("%s:%s: time = %f\n",
                                        __FILE__, __func__,
                                        device->fix->time);

                if (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET)
                        g_printf("%s:%s: lat = %f, long = %f\n",
                                        __FILE__,
                                        __func__,
                                        device->fix->latitude,
                                        device->fix->longitude);

                if (device->fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET)
                        g_printf("%s:%s: alt = %f\n",
                                        __FILE__, __func__,
                                        device->fix->altitude);

                if (device->fix->fields & LOCATION_GPS_DEVICE_SPEED_SET)
                        g_printf("%s:%s: speed = %f\n",
                                        __FILE__, __func__,
                                        device->fix->speed);

                if (device->fix->fields & LOCATION_GPS_DEVICE_TRACK_SET)
                        g_printf("%s:%s: track = %f\n",
                                        __FILE__, __func__,
                                        device->fix->track);
                if (device->fix->fields & LOCATION_GPS_DEVICE_CLIMB_SET)
                        g_printf( "%s:%s: climb = %f\n",
                                        __FILE__, __func__,
                                        device->fix->climb);

                g_printf("%s:%s Accuracy values:\n", __FILE__, __func__);
                g_printf("\tept = %e, eph = %e, epv = %e, epd = %e, "
                                "eps = %e, epc = %e\n",
                                device->fix->ept,
                                device->fix->eph,
                                device->fix->epv,
                                device->fix->epd,
                                device->fix->eps,
                                device->fix->epc);
#endif
	} /* device->fix */

#ifndef __arm__
        g_printf("%s:%s: Satellites in view: %d\n",
                        __FILE__, __func__,
                        device->satellites_in_view);

        g_printf("%s:%s: Satellites in use: %d\n", __FILE__, __func__,
                        device->satellites_in_use);

        if (device && device->cell_info) {
                if (device->cell_info->flags & LOCATION_CELL_INFO_GSM_CELL_INFO_SET)
                        g_printf( "Mobile Coutry Code GSM: %d\n",
                                        device->cell_info->gsm_cell_info.mcc);

                if (device->cell_info->flags & LOCATION_CELL_INFO_WCDMA_CELL_INFO_SET)
                        g_printf( "Mobile Coutry Code WCDMA: %d\n",
                                        device->cell_info->wcdma_cell_info.mcc);
        } 
#endif
}

/* -------------------------------------------------------------------- */

static gboolean
on_background_expose(GtkWidget *widget, GdkEventExpose *expose_event, gpointer data)
{
    /* GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data); */
    cairo_t *cr = NULL;

    cr = gdk_cairo_create(widget->window);

    if (!cr) 
        return FALSE;

    cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
    gdk_cairo_region(cr, expose_event->region);
    cairo_fill(cr);

    draw_background(cr, &expose_event->area, 3, 0.6);

    cairo_destroy(cr);

    return FALSE;
}

static gboolean
on_button_expose(GtkWidget *widget, GdkEventExpose *expose_event, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);

    cairo_t *cr = NULL;
    cr = gdk_cairo_create(widget->window);

    if (!cr) return FALSE;

    /* button background */
    /* If theme images are not loaded draw with cairo */
    if (!theme_pixbufs_loaded(self))
    {
	if (((widget == self->button_rec) && (self->rec_button_down)) ||
	   ((widget == self->button_stop) && (self->stop_button_down)))
                draw_background(cr, &expose_event->area, 0, 0.6);
            else
                draw_background(cr, &expose_event->area, 0, 0.8);
    } else {
        if (((widget == self->button_rec) && (self->rec_button_down)) ||
            ((widget == self->button_stop) && (self->stop_button_down))) 
                draw_pixbuf(cr, self->button_pressed_bg,
			    widget->allocation.x + widget->allocation.width/2,
			    widget->allocation.y + widget->allocation.height/2,
			    FALSE);
        else if (((widget == self->button_rec) && (! self->rec_button_enabled)) ||
                ((widget == self->button_stop) && (! self->stop_button_enabled))) 
                draw_pixbuf(cr, self->button_bg,
			    widget->allocation.x + widget->allocation.width/2,
			    widget->allocation.y + widget->allocation.height/2,
			    TRUE);
        else
                draw_pixbuf(cr, self->button_bg,
			    widget->allocation.x + widget->allocation.width/2,
			    widget->allocation.y + widget->allocation.height/2,
			    FALSE);
    }
    /* button icons */
    if (widget == self->button_stop)
            draw_pixbuf(cr, self->stop_icon,
			    widget->allocation.x + widget->allocation.width/2,
			    widget->allocation.y + widget->allocation.height/2,
			    ! self->stop_button_enabled);
    else if ((widget == self->button_rec) && (self->pause_button_visible))
            draw_pixbuf(cr, self->pause_icon,
			    widget->allocation.x + widget->allocation.width/2,
			    widget->allocation.y + widget->allocation.height/2,
			    ! self->rec_button_enabled);
    else if ((widget == self->button_rec) && (! self->pause_button_visible))
            draw_pixbuf(cr, self->rec_icon,
			    widget->allocation.x + widget->allocation.width/2,
			    widget->allocation.y + widget->allocation.height/2,
			    ! self->rec_button_enabled);

    cairo_destroy(cr);

    return FALSE;
}

/* -------------------------------------------------------------------- */
/* Callback for hardware D-BUS events */

void hw_event_handler(osso_hw_state_t *state, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
    gboolean should_stop_recording = FALSE;
#ifdef DEBUG
    if (self->logfile) {
        g_fprintf(self->logfile, "%s -- enter\n", __func__);
        g_fprintf(self->logfile, "  hw event received - ");
        fflush(self->logfile);
    }
#endif

    if (state->shutdown_ind)
    {
        hildon_banner_show_information(NULL, NULL, 
                                       _("Shutdown event!"));
        should_stop_recording = TRUE;
#ifdef DEBUG
        if (self->logfile) {
            g_fprintf(self->logfile, "  Shutdown event!\n");
            fflush(self->logfile);
        }
#endif
    }
    if (state->save_unsaved_data_ind)
    {
        hildon_banner_show_information(NULL, NULL, 
                                       _("Must save unsaved data event!"));
        should_stop_recording = TRUE;
#ifdef DEBUG
        if (self->logfile) {
            g_fprintf(self->logfile, "  Must save unsaved data event!\n");
            fflush(self->logfile);
        }
#endif
    }
#ifdef DEBUG
    if (state->memory_low_ind)
    {
/*        hildon_banner_show_information(NULL, NULL, 
                                       "Memory low event!"); */
        if (self->logfile) {
            g_fprintf(self->logfile, "  Memory low event!\n");
            fflush(self->logfile);
        }
    }
    if (state->system_inactivity_ind)
    {
/*        hildon_banner_show_information(NULL, NULL, 
                                       "Minimize application inactivity event!"); */
        if (self->logfile) {
            g_fprintf(self->logfile, "  Minimize application inactivity event!\n");
            fflush(self->logfile);
        }
    }
#endif
    /* shall we stop recording and flush the log? */
    if ((self->app_state != APP_STATE_IDLE) && should_stop_recording) {
        /* let's prettend that the user pressed stop */
        handle_button_press(self->button_stop, self);
    }
}

static DBusHandlerResult
dbus_signal_filter(DBusConnection *conn, DBusMessage *message, gpointer data)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
    char *display_mode;

    /* if idle, we do not care if screen is on or off */
    if (self->app_state == APP_STATE_IDLE)
        return DBUS_HANDLER_RESULT_HANDLED;
    if (dbus_message_is_signal(message, MCE_SIGNAL_IF, MCE_DISPLAY_SIG)) {
        if (dbus_message_get_args(message, NULL, DBUS_TYPE_STRING, &display_mode,
				  DBUS_TYPE_INVALID)) {
            if (!g_strcmp0(display_mode, MCE_DISPLAY_OFF_STRING)) {
                self->display_status = FALSE;
	    } else {
                self->display_status = TRUE;
		gps_data_logger_update_distance(self);
	    }
#ifndef __arm__
            g_print("Display status: %d\n", self->display_status);
#endif
	}
	return DBUS_HANDLER_RESULT_HANDLED;
    } else {
        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
    }
}

static void
gps_data_logger_widget_init (GpsDataLoggerWidget * self)
{
    GtkWidget *vbox, *hbox_status, *hbox_buttons, *hbox_distance;
    GtkWidget *status_text;
    GdkScreen *screen;
    GdkColormap *colormap;
    osso_return_t result;
    GString *version = NULL ;
    GString *tmp = NULL;
    DBusError error;

    version = g_string_new(getenv("OSSO_PRODUCT_RELEASE_VERSION"));
#ifndef __arm__
    g_print("SW version: %s\n", version->str);
#endif
    self->osso_context = osso_initialize(OSSO_GDL_SERVICE, "0.0.1", TRUE, NULL);
    /* Check that initialization was ok */
    if (self->osso_context == NULL) {
#ifndef __arm__
        g_print("init : osso_initialize failed.\n");
#endif
        hildon_banner_show_information(NULL,
			NULL,
			_("osso_initialize failed"));
    }
    self->file = NULL;
    self->filtered_file = NULL;
    self->path = NULL;
#ifdef DEBUG
    self->logfile = NULL;
#endif
    self->display_status = TRUE;
    self->app_state = APP_STATE_IDLE;

    /* setup dbus for display state information */
    dbus_error_init(&error);
    self->dbus_conn = hd_home_plugin_item_get_dbus_connection(
		      HD_HOME_PLUGIN_ITEM(self), DBUS_BUS_SYSTEM, &error);
    if (dbus_error_is_set (&error)) {
#ifndef __arm__
        g_print("init : hd_home_plugin_item_get_dbus_connection failed.\n");
#endif
        hildon_banner_show_information(NULL,
			NULL,
			_("Can not get DBus connection!"));
	dbus_error_free (&error);
    } else {
        dbus_connection_setup_with_g_main(self->dbus_conn, NULL);
	dbus_bus_add_match (self->dbus_conn, MCE_MATCH_RULE, &error);
	if (dbus_error_is_set (&error)) {
#ifndef __arm__
            g_print("init : dbus_bus_add_match for DISPLAY_MODE_SIGNAL failed.\n");
#endif
            hildon_banner_show_information(NULL,
	    		    NULL,
			    _("Could not add DBus match!"));
	    dbus_error_free (&error);
        } else {
	    if (!dbus_connection_add_filter(self->dbus_conn, dbus_signal_filter,
				            self, NULL)) {
#ifndef __arm__
                g_print("init : dbus_connection_add_filter failed.\n");
#endif
                hildon_banner_show_information(NULL,
  	    		        NULL,
			        _("Could not add DBus filter!"));
	    }
	}
    }
    
    self->file = NULL;			    
    
    /* Preload all icon pixmaps */
    load_icons(self);
    load_theme_pixbufs(self);

    /* Needed for transparency */
    screen = gtk_widget_get_screen(GTK_WIDGET(self));
    colormap = gdk_screen_get_rgba_colormap(screen);
    gtk_widget_set_colormap(GTK_WIDGET(self), colormap);

    self->background = gtk_event_box_new();
    gtk_widget_set_app_paintable(self->background, TRUE);
    gtk_container_add(GTK_CONTAINER(self), self->background);

    vbox = gtk_vbox_new (FALSE, HILDON_MARGIN_HALF);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), HILDON_MARGIN_DEFAULT);
    gtk_container_add(GTK_CONTAINER(self->background), vbox);

    self->rec_button_down = FALSE;
    self->rec_button_enabled = TRUE;
    self->button_rec = gtk_event_box_new();
    if (theme_pixbufs_loaded(self))
        gtk_widget_set_size_request(self->button_rec, 
		        gdk_pixbuf_get_width(self->button_bg),
                        gdk_pixbuf_get_height(self->button_bg));
    else
        gtk_widget_set_size_request(self->button_rec,96,96); 
    gtk_widget_set_app_paintable(self->button_rec, TRUE);
    gtk_event_box_set_visible_window(GTK_EVENT_BOX(self->button_rec), FALSE);
    
    self->stop_button_down = FALSE;
    self->stop_button_enabled = FALSE;
    self->button_stop = gtk_event_box_new();
    if (theme_pixbufs_loaded(self))
        gtk_widget_set_size_request(self->button_stop,
		        gdk_pixbuf_get_width(self->button_bg),
                        gdk_pixbuf_get_height(self->button_bg));
    else
        gtk_widget_set_size_request(self->button_stop,96,96);
    gtk_widget_set_app_paintable(self->button_stop, TRUE);
    gtk_event_box_set_visible_window(GTK_EVENT_BOX(self->button_stop), FALSE);

    self->gps_status = gtk_image_new();
    self->auto_pause_status = gtk_image_new();
    status_text = gtk_label_new(_("Status: "));

    hbox_status = gtk_hbox_new (FALSE, 3);
    hbox_buttons = gtk_hbox_new (FALSE, 2);

    self->distance_label = gtk_label_new(_("Distance: "));
    self->distance_value_string = gtk_label_new(_("0 m"));
    self->distance = 0;
    hbox_distance = gtk_hbox_new (FALSE, 2);

    gtk_box_pack_start (GTK_BOX (hbox_status), status_text, FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_status), self->gps_status, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_status), self->auto_pause_status, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_distance), self->distance_label, FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_distance), self->distance_value_string, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_rec, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_stop, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox_status, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox_distance, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox_buttons, FALSE, FALSE, 0);
     
#ifndef __arm__
    g_print("init : calling update_status_and_buttons\n");
#endif
    update_status_and_buttons(self);

    gtk_widget_show_all (self->background);
    
    /* Set the resizing behavior */
    /*
    hd_home_widget_item_set_resize_type (HD_HOME_PLUGIN_ITEM (home_widget),
					 HD_HOME_PLUGIN_ITEM_RESIZE_BOTH);
    */

    self->control = location_gpsd_control_get_default();

    self->settings = settings_load();
    apply_settings(self);

    self->device  = g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);
    
    /* location signals */
    g_signal_connect(self->control, "error", G_CALLBACK(on_error), self);
    g_signal_connect(self->control, "gpsd-running", G_CALLBACK(on_start),
                                                                   self);
    g_signal_connect(self->control, "gpsd-stopped", G_CALLBACK(on_stop),
                                                                   self);
    g_signal_connect(self->device,  "changed", G_CALLBACK(on_changed),
    		    self);

    /* widget settings */
    hd_home_plugin_item_set_settings (HD_HOME_PLUGIN_ITEM (self), TRUE);
    g_signal_connect (self, "show-settings", G_CALLBACK (on_settings_dialog_show),
		    self);

    tmp = g_string_new("1.");
    g_string_truncate(version, 2);
    if (g_string_equal(version, tmp)) {
        /* On the first release of Fremantle all the button events are
         * going to the background, so let's register those too */
        g_signal_connect(G_OBJECT(self->background), "button-release-event",
                         G_CALLBACK(on_button_release), self);
        g_signal_connect(G_OBJECT(self->background), "button-press-event",
                         G_CALLBACK(on_button_press), self);
    } else {
        /* button signals */
        g_signal_connect(G_OBJECT(self->button_rec), "button-release-event",
		         G_CALLBACK(on_button_release), self);
        g_signal_connect(G_OBJECT(self->button_rec), "button-press-event",
		         G_CALLBACK(on_button_press), self);
        g_signal_connect(G_OBJECT(self->button_stop), "button-release-event",
		         G_CALLBACK(on_button_release), self);
        g_signal_connect(G_OBJECT(self->button_stop), "button-press-event",
		         G_CALLBACK(on_button_press), self);
    }
    
    g_signal_connect(G_OBJECT(self->background), "leave-notify-event",
                    G_CALLBACK(on_widget_leave), self);
    /* expose events */
    g_signal_connect(self->background, "expose-event", 
		     G_CALLBACK(on_background_expose),
		     (gpointer)self);
    g_signal_connect(self->button_rec, "expose-event", 
                     G_CALLBACK(on_button_expose),
                     (gpointer)self);
    g_signal_connect(self->button_stop, "expose-event", 
                     G_CALLBACK(on_button_expose),
                     (gpointer)self);
    g_signal_connect(G_OBJECT(self->background), "style-set",
                     G_CALLBACK(on_style_set), (gpointer)self);
#ifdef DEBUG
    debuglog_start(self);
#endif
    if (self->osso_context) {
        /* Add handler for hardware D-BUS messages */
        result = osso_hw_set_event_cb( self->osso_context,
            NULL, hw_event_handler, (gpointer)self );

        if (result != OSSO_OK)
        {
#ifndef __arm__
            g_print("Error setting HW state callback (%d)\n", result);
#endif
        }
    }
    g_string_free(version, TRUE);
    g_string_free(tmp, TRUE);

}

static void
gps_data_logger_widget_finalize (GObject *object)
{
    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (object);
    /* Close the logfile if it is open */
    logfile_close(self);
#ifdef DEBUG
    debuglog_close(self);
#endif
    g_object_unref(self->device);
    g_object_unref(self->control);
    if (self->settings_dialog) {
        gtk_widget_destroy (self->settings_dialog);
        self->settings_dialog = NULL;
    }
    settings_free(self->settings);
    g_free(self->settings);
    free_icons(self);
    free_theme_pixbufs(self);
    /* Deinitialize OSSO */
    if (self->osso_context) {
        osso_deinitialize(self->osso_context);
        self->osso_context = NULL;
    }

    dbus_bus_remove_match (self->dbus_conn, MCE_MATCH_RULE, NULL);
    dbus_connection_remove_filter (self->dbus_conn, dbus_signal_filter, self);
    dbus_connection_close (self->dbus_conn);
    dbus_connection_unref (self->dbus_conn);

		    
    /* Call the base class's implementation: */
    G_OBJECT_CLASS(gps_data_logger_widget_parent_class)->finalize (object);
}

static void
gps_data_logger_widget_class_init (GpsDataLoggerWidgetClass * klass)
{
    GObjectClass         *object_class;
    object_class = G_OBJECT_CLASS (klass);
    object_class->finalize = gps_data_logger_widget_finalize;
}

static void
gps_data_logger_widget_class_finalize (GpsDataLoggerWidgetClass * klass)
{
}

